// LabWindows/CVI Example using rsspecan VXIpnp driver
// Notes:      To run the example you need to install rsspecan VXI Plug&Play
//             instrument driver version 3.8.0 or later.
//             Default locations of instrument driver files on 32bit Windows:
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\rsspecan\rsspecan.fp
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\include\rsspecan.h
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\lib\msc\rsspecan.lib
//             Default locations of instrument driver files on 64bit Windows:
//             C:\Program Files\IVI Foundation\VISA\Win64\rsspecan\rsspecan.fp
//             C:\Program Files\IVI Foundation\VISA\Win64\include\rsspecan.h
//             C:\Program Files\IVI Foundation\VISA\Win64\lib\msc\rsspecan.lib

#include "rsspecan.h"
#include <utility.h>
#include <ansi_c.h>

// error checking macro
#define CHECKERR(fCal) \
	status = (fCal);if (status < VI_SUCCESS) goto ERROR_HANDLING;

#define ARRAY_SIZES 30000

    

int main()
{
	ViSession specan;
	ViStatus status = VI_SUCCESS;
    ViInt32 errorCode;
	ViChar response[1024];
    ViChar  error_message [1024];
    ViChar  error_buffer  [16384];
    ViChar* p2buf;
	ViReal64 trace[ARRAY_SIZES];
	ViInt32 count;
    ViInt32 opc;
    ViReal64 markerX, markerY;
       

	//-----------------------------------------------------------
	//Initialization:
	//-----------------------------------------------------------
    SetBreakOnLibraryErrors (0); // Don't show Pupup messages with errors from libraries.
    CHECKERR(rsspecan_init("TCPIP::192.168.1.101::INSTR", VI_TRUE, VI_FALSE, &specan)); // Adjust the VISA Resource string to fit your instrument
	CHECKERR(rsspecan_IDQueryResponse(specan, 1024, response));
	CHECKERR(rsspecan_reset(specan)); // Reset the instrument, clear the Error queue
	CHECKERR(rsspecan_ConfigureDisplayUpdate(specan,VI_TRUE)); // Display update ON - switch OFF after debugging
	//-----------------------------------------------------------
	// Basic Settings:
	//---------------------------- -------------------------------

    CHECKERR(rsspecan_ConfigureReferenceLevel(specan, 0, 10.0)); // Setting the Reference Level
	CHECKERR(rsspecan_ConfigureFrequencyCenterSpan(specan, 0, 3E9, 200E6)); // Setting the center frequency and span
	CHECKERR(rsspecan_ConfigureSweepCoupling (specan, 0, RSSPECAN_VAL_COUPLING_RBW, 100E3)); // Setting the RBW
    CHECKERR(rsspecan_ConfigureSweepCoupling (specan, 0, RSSPECAN_VAL_COUPLING_VBW, 300E3)); // Setting the VBW
    CHECKERR(rsspecan_ConfigureSweepPoints (specan, 0, 10001)); // Setting the sweep points
	CHECKERR(rsspecan_QueryOPC(specan, &opc)); //Using *OPC? query waits until all the instrument settings are finished
	// -----------------------------------------------------------
	// SyncPoint 'SettingsApplied' - all the settings were applied
	// -----------------------------------------------------------
    CHECKERR(rsspecan_Initiate(specan, 0, 2000)); // Start the sweep, wait for it to finish with timeout 2000 ms
	// -----------------------------------------------------------
	// SyncPoint 'AcquisitionFinished' - the results are ready
	// -----------------------------------------------------------
	// Fetching the trace - only Binary format is available in drivers
	// -----------------------------------------------------------
	printf("Fetching trace in Binary format... ");
	CHECKERR(rsspecan_FetchYTrace (specan, 0, 1, ARRAY_SIZES, &count, trace));
    printf("Sweep points count: %d\n", count);
    // -----------------------------------------------------------
    // Setting the marker to max and querying the X and Y
    // -----------------------------------------------------------
    CHECKERR(rsspecan_MarkerSearch(specan,0,1,RSSPECAN_VAL_MARKER_SEARCH_HIGHEST)); // Set the marker to the maximum point of the entire trace
    CHECKERR(rsspecan_QueryMarker (specan, 0, 1, &markerX, &markerY)); // Query marker X and Y coordinates
    printf("Marker Frequency %0.3f MHz, Level %0.2f dBm", markerX/1E6, markerY);
	// -----------------------------------------------------------
	// Making an instrument screenshot and transferring the file to the PC
	// -----------------------------------------------------------
	CHECKERR(rsspecan_ConfigureHardcopyDevice (specan, 1, RSSPECAN_VAL_HCOPY_DEST_PNG, RSSPECAN_VAL_HCOPY_DEVICE_ORIENT_LAND)); // Set the screenshot format
	CHECKERR(rsspecan_HardcopySetFileName(specan, "c:\\temp\\Dev_Screenshot.png")); // Set the screenshot path
	CHECKERR(rsspecan_HardcopyPrint (specan, 1, RSSPECAN_VAL_HCOP_ALL)); // Make the screenshot now
    CHECKERR(rsspecan_ReadToFileFromInstrument(specan, "c:\\temp\\Dev_Screenshot.png", "c:\\temp\\PC_Screenshot.png")); // Copy the file from instrument to the PC
	printf("\nScreenshot file saved to PC 'c:\\Temp\\PC_Screenshot.png'");

ERROR_HANDLING:
	if (status == VI_SUCCESS)
	{
		; // all went OK
	}
	else if (status < VI_SUCCESS)
    {
        rsspecan_error_message (specan, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsspecan_error_query (specan, &errorCode, error_message);
        
        if (errorCode != 0)
        {
            sprintf (p2buf, "Secondary Error: Instrument is reporting the following error (SYST:ERR?): %d, %s\n", errorCode, error_message);
        }
        
        printf ("\nERROR!\n\n");
        printf (error_buffer);
    }
        
    rsspecan_close (specan);
    
    
    printf ("\nPress ENTER to continue...");
    (void) getchar();
        
    return status;
}
